
/***VVID*****************************************************************/
/*									*/
/*  Project	: CONZEPT 16 PGIF					*/
/*  Project ID	: C16.PGIF						*/
/*									*/
/*  Program	: CONZEPT 16 PROGRAMMING INTERFACE - HEADER		*/
/*  Revision	: 4.4.00						*/
/*  FileName	: c16_dll.h						*/
/*									*/
/*  Last User	: ORIGINAL						*/
/*									*/
/***VVID*****************************************************************/

#if !defined(INCL_C16_DLL)
#define INCL_C16_DLL

#ifdef __cplusplus
extern "C"
{
#endif

/****************************************************************************/
/*									    */
/*  BASIC TYPES (only for 32/64-Bit environment)			    */
/*									    */
/****************************************************************************/

#ifndef INCL_S_DEFINE			    /* - internal -		    */

typedef unsigned short		vWORD;	    /* 16 bit unsigned		    */
typedef signed short		vWORDs;     /* 16 bit signed		    */
typedef unsigned int		vLONG;	    /* 32 bit unsigned		    */
typedef signed int		vLONGs;     /* 32 bit signed		    */

typedef unsigned char		vCHAR;	    /* 8 bit unsigned		    */
typedef unsigned char		vBYTE;	    /* 8 bit unsigned		    */
typedef signed char		vBYTEs;     /* 8 bit signed		    */
typedef unsigned int		vINT;	    /* natural unsigned 	    */
typedef signed int		vINTs;	    /* natural signed		    */

typedef vINT			vBOOL;	    /* native boolean		    */
typedef vBYTE			vBOOL8;     /*	8 bit boolean		    */
typedef vWORD			vBOOL16;    /* 16 bit boolean		    */
typedef vLONG			vBOOL32;    /* 32 bit boolean		    */

typedef double			vFLOAT;     /* 64-bit floating point	    */
typedef unsigned long		vPTR;	    /* 32/64-bit pointer	    */

typedef signed int		vNHANDLE;   /* 16/32-bit handle 	    */
typedef vPTR			vPHANDLE;   /* 32/64-bit handle 	    */
typedef vINTs			vERROR;     /* error result		    */
typedef vLONG			vFLAGS;     /* flag type		    */

typedef struct				    /* date type		    */
{					    /* y = year, m = month	    */
  vBYTE y,m,d,c;			    /* d = day,  c = <unused>	    */
}
vDATE;

typedef struct				    /* time type		    */
{					    /* h = hour, m = minute	    */
  vBYTE h,m,s,c;			    /* s = seconds, c = 1/100s	    */
}
vTIME;

#endif

/****************************************************************************/
/*									    */
/*  ERROR VALUES							    */
/*									    */
/****************************************************************************/

#define C16ERR_OK			 0

#define C16ERR_LDM_BASE 		-100
#define C16ERR_LDM_BEGIN		-101
#define C16ERR_LDM_END			-190
#define C16ERR_LDM_INVALID		-191
#define C16ERR_LDM_TYPE_INVALID 	-192

#define C16ERR_INIT_FAILED		-201
#define C16ERR_MODULE_HDL_INVALID	-202
#define C16ERR_INSTANCE_HDL_INVALID	-203
#define C16ERR_BUFFER_HDL_INVALID	-204
#define C16ERR_FUNCTION_NOT_AVAILABLE	-205
#define C16ERR_NO_MULTIPLE_INSTANCES	-206
#define C16ERR_TEXT_HDL_INVALID 	-207
#define C16ERR_SEL_HDL_INVALID		-208

#define C16ERR_OUT_OF_MEMORY		-301
#define C16ERR_COMM_FAILED		-302
#define C16ERR_SERVER_OP		-303

#define C16ERR_NO_SERVER_CONNECTION	-401
#define C16ERR_AREA_NOT_FOUND		-402
#define C16ERR_AREA_OPEN		-403
#define C16ERR_AREA_LOCKED		-404
#define C16ERR_AREA_IN_USE		-405
#define C16ERR_AREA_TYPE		-406
#define C16ERR_AREA_PASSWORD		-407
#define C16ERR_SERVER_USER_LIMIT	-408
#define C16ERR_SERVER_START		-409
#define C16ERR_USER_PASSWORD		-410
#define C16ERR_DATA_STRUCTURE		-411
#define C16ERR_PGIF_USER_LIMIT		-412
#define C16ERR_USER_INVALID		-413
#define C16ERR_USER_SELF		-414
#define C16ERR_USER_SHM 		-415

#define C16ERR_INFO_BLOCKSIZE_INVALID	-501

#define C16ERR_NO_FILE			-601
#define C16ERR_NO_SBR			-602
#define C16ERR_NO_FLD			-603
#define C16ERR_NO_KEY			-604
#define C16ERR_NO_KEY_FLD		-605
#define C16ERR_NO_LINK			-606
#define C16ERR_NO_LINK_FLD		-607
#define C16ERR_NO_FLD_DATA		-608
#define C16ERR_LINK_INVALID		-609

#define C16ERR_NO_ARGUMENT		-610
#define C16ERR_ARGUMENT_UNDEFINED	-611
#define C16ERR_ARGUMENT_READ_ONLY	-612

#define C16ERR_TEXT_NAME_INVALID	-701
#define C16ERR_TEXT_RIGHTS		-702
#define C16ERR_TEXT_UNDEFINED		-703

#define C16ERR_SEL_FILE_INVALID 	-801
#define C16ERR_SEL_NAME_INVALID 	-802

/****************************************************************************/
/*									    */
/*  CONSTANTS								    */
/*									    */
/****************************************************************************/

#define _TypeAlpha		    1
#define _TypeDate		    2
#define _TypeByte		    3
#define _TypeWord		    4
#define _TypeInt		    7
#define _TypeFloat		    9
#define _TypeLogic		    10
#define _TypeTime		    11
#define _TypeFix		    12

#define _ArgOptVAR		    0x40
#define _ArgOptARRAY		    0x80

#define _KeyFldAttrUpperCase	    0x02
#define _KeyFldAttrUmlaut	    0x04
#define _KeyFldAttrSpecialChars     0x08
#define _KeyFldAttrSoundex1	    0x10
#define _KeyFldAttrSoundex2	    0x20
#define _KeyFldAttrReverse	    0x40

#define _LinkFldAttrUpperCase	    0x02
#define _LinkFldAttrUmlaut	    0x04
#define _LinkFldAttrSpecialChars    0x08
#define _LinkFldAttrSoundex1	    0x10
#define _LinkFldAttrSoundex2	    0x20
#define _LinkFldAttrReverse	    0x40
#define _LinkFldAttrPosition	    0x80

#define _FmtNone		    0x00
#define _FmtTSep		    0x01
#define _FmtZeroSupr		    0x02
#define _FmtPlusSign		    0x04
#define _FmtDateLongYear	    0x08
#define _FmtTimeSeconds 	    0x10
#define _FmtTimeHSeconds	    0x20

#define _FldExc_Binary		    0x00
#define _FldExc_Character	    0x01
#define _FldExc_RawData 	    0x02
#define _FldExc_CharSet_C16	    0
#define _FldExc_CharSet_OEM	    1
#define _FldExc_CharSet_ANSI	    2

#define _rOk			    0
#define _rLocked		    1
#define _rMultiKey		    2
#define _rNoKey 		    3
#define _rLastRec		    4
#define _rNoRec 		    5
#define _rExists		    6
#define _rNoLock		    7

#define _RecFirst		    0x01
#define _RecLast		    0x02
#define _RecPrev		    0x03
#define _RecNext		    0x04
#define _RecPos 		    0x05
#define _RecLock		    0x08
#define _RecForceLock		    0x10
#define _RecUnlock		    0x20
#define _RecCheckLock		    0x18
#define _RecTest		    0x40
#define _RecNoLoad		    0x80

#define _RecCount		    0
#define _RecCountPos		    5
#define _RecCountNext		    6
#define _RecID			    1
#define _RecLen 		    2
#define _RecLockedBy		    3
#define _RecGetPos		    4

#define _TextFirst		    0x00001
#define _TextLast		    0x00002
#define _TextPrev		    0x00003
#define _TextNext		    0x00004
#define _TextLock		    0x00008
#define _TextUnlock		    0x00020
#define _TextBinary		    0x00100
#define _TextAppend		    0x00200
#define _TextSingleLine 	    0x00400

#define _TextEolNull		    0x00000
#define _TextEolCR		    0x01000
#define _TextEolLF		    0x02000
#define _TextEolCRLF		    0x03000
#define _Text_CharSet_C16	    0x00000
#define _Text_CharSet_OEM	    0x10000
#define _Text_CharSet_ANSI	    0x20000

#define _PgifType_Client	    0x0001	/* Client-type		    */
#define _PgifType_Protocol_16	    0x0080	/* 16-bit protocols	    */
#define _PgifType_Win32 	    0x0100	/* Windows 32-Bit	    */
#define _PgifType_OS2		    0x0200	/* OS/2 32-Bit		    */

#define _ProtocolSPX		    1		/* IPX/SPX		    */
#define _ProtocolTCP		    2		/* TCP/IP		    */
#define _ProtocolNPS		    4		/* Named Pipes		    */
#define _ProtocolSHM		    5		/* Shared Memory	    */

#define _AreaOpen		    0x0001	/* area is open 	    */
#define _AreaUpdate		    0x0002	/* area update running	    */
#define _AreaBackup		    0x0004	/* area backup event	    */
#define _AreaLock		    0x0008	/* area is locked	    */
#define _AreaDiag		    0x0020	/* area diagnose running    */
#define _AreaRecover		    0x0080	/* area recover running     */
#define _AreaOptimize		    0x0200	/* area optimize running    */
#define _AreaReadOnly		    0x0400	/* area is read-only	    */

/****************************************************************************/
/*									    */
/*  STRUCTURES								    */
/*									    */
/****************************************************************************/

/* file information structure */
typedef struct
{
  vINT			InfoSize;		/* size of information block */

  vINT			FileNumber;		/* filenumber		    */
  vINT			FileMaster;		/* master filenumber	    */
  vINT			FileSbrCount;		/* # of subrecords	    */
  vINT			FileKeyCount;		/* # of keys		    */
  vINT			FileLinkCount;		/* # of links		    */
  vCHAR 		FileName[24];		/* filename		    */
}
vC16_FileInfo;

/* subrecord information structure */
typedef struct
{
  vINT			InfoSize;		/* size of information block */

  vINT			SbrFileNumber;		/* subrecord filenumber     */
  vINT			SbrNumber;		/* subrecord number	    */
  vBOOL 		SbrStatus;		/* subrecord active status  */
  vINT			SbrFldCount;		/* # of fields		    */
  vCHAR 		SbrName[24];		/* subrecord name	    */
}
vC16_SbrInfo;

/* field information structure */
typedef struct
{
  vINT			InfoSize;		/* size of information block */

  vINT			FldFileNumber;		/* field filenumber	    */
  vINT			FldSbrNumber;		/* field subrecord number   */
  vINT			FldNumber;		/* field number 	    */
  vBYTE 		FldType;		/* subrecord active status  */
  vBYTE 		FldLen; 		/* # of fields		    */
  vBYTE 		FldInputRight;		/* # of fields		    */
  vBYTE 		FldOutputRight; 	/* # of fields		    */
  vCHAR 		FldName[24];		/* field name		    */
}
vC16_FldInfo;

/* key information structure */
typedef struct
{
  vINT			InfoSize;		/* size of information block */

  vINT			KeyFileNumber;		/* key filenumber	    */
  vINT			KeyNumber;		/* key number		    */
  vINT			KeyFldCount;		/* # of fields		    */
  vBOOL 		KeyIsUnique;		/* # of fields		    */
  vCHAR 		KeyName[24];		/* key name		    */
}
vC16_KeyInfo;

/* key field information structure */
typedef struct
{
  vINT			InfoSize;		/* size of information block */

  vINT			KeyFldFileNumber;	/* keyfield filenumber	    */
  vINT			KeyFldSbrNumber;	/* keyfield subrecord no.   */
  vINT			KeyFldNumber;		/* keyfield field number    */
  vBYTE 		KeyFldType;		/* keyfield type	    */
  vBYTE 		KeyFldAttributes;	/* keyfield attributes	    */
  vBYTE 		KeyFldMaxLen;		/* max. length of keyfield  */
  vBYTE 		reserved;		/*			    */
  vCHAR 		KeyFldName[24]; 	/* keyfield name	    */
}
vC16_KeyFldInfo;

/* link information structure */
typedef struct
{
  vINT			InfoSize;		/* size of information block */

  vINT			LinkFileNumber; 	/* link filenumber	    */
  vINT			LinkNumber;		/* link number		    */
  vINT			LinkFldCount;		/* # of fields		    */
  vINT			LinkDestFileNumber;	/* link destination file no. */
  vINT			LinkDestKeyNumber;	/* link destination key no. */
  vCHAR 		LinkName[24];		/* link name		    */
}
vC16_LinkInfo;

/* link field information structure */
typedef struct
{
  vINT			InfoSize;		/* size of information block */

  vINT			LinkFldFileNumber;	/* linkfield filenumber     */
  vINT			LinkFldSbrNumber;	/* linkfield subrecord no.  */
  vINT			LinkFldNumber;		/* linkfield field number   */
  vBYTE 		LinkFldType;		/* linkfield type	    */
  vBYTE 		LinkFldAttributes;	/* linkfield attributes     */
  vBYTE 		LinkFldMaxLen;		/* max. length of linkfield */
  vBYTE 		reserved;		/*			    */
  vCHAR 		LinkFldName[24];	/* linkfield name	    */
}
vC16_LinkFldInfo;

/* text header information structure */
typedef struct
{
  vINT			InfoSize;		/* size of information block */

  /* read only elements */

  vCHAR 		TextName[24];		/* name of text 	    */
  vCHAR 		TextPrivUser[24];	/* private text user name   */
  vCHAR 		TextCreateUser[24];	/* creation user name	    */
  vCHAR 		TextModifiedUser[24];	/* last modification user name */

  vDATE 		TextCreateDate; 	/* creation date	    */
  vTIME 		TextCreateTime; 	/* creation time	    */
  vDATE 		TextModifiedDate;	/* last modification date   */
  vTIME 		TextModifiedTime;	/* last modification time   */

  vLONG 		TextSize;		/* size in bytes	    */
  vLONG 		TextLines;		/* number of lines	    */

  /* changeable elements */

  vCHAR 		TextGroup[24];		/* text group name	    */
  vINT			TextReadPL;		/* read permission level    */
  vINT			TextWritePL;		/* write permission level   */
  vBOOL 		TextPrivate;		/* text is private	    */
  vBOOL 		TextEncrypted;		/* text is encrypted	    */
}
vC16_TextInfo;

/* interface information structure */
typedef struct
{
  vINT			InfoSize;		/* size of information block */

  vINT			PgifType;		/* type of interface	    */
  vCHAR 		PgifLicense[20];	/* license number	    */
  vCHAR 		PgifRelease[8]; 	/* release info 	    */
  vINT			PgifUserLimit;		/* user limit		    */
  vINT			PgifMemory;		/* current allocated memory */
  vINT			PgifMemoryPeak; 	/* peak allocated memory    */
}
vC16_PgifInfo;

/* server information structure */
typedef struct
{
  vINT			InfoSize;		/* size of information block */

  vCHAR 		ServerName[64]; 	/* server host name	    */
  vCHAR 		ServerType[48]; 	/* type of server	    */
  vCHAR 		ServerLicense[20];	/* server license	    */
  vCHAR 		ServerRelease[8];	/* server release	    */
  vINT			ServerProtocol; 	/* used protocol	    */
  vINT			ServerUserLimit;	/* server user limit	    */
}
vC16_ServerInfo;

/* data area information structure */
typedef struct
{
  vINT			InfoSize;		/* size of information block */

  vINT			UserID; 		/* current user id	    */
  vINT			UserNumber;		/* current user number	    */

  vINT			AreaID; 		/* area release info	    */
  vLONG 		AreaSize;		/* size of area (KB)	    */
  vLONG 		AreaFree;		/* free area space (KB)     */
  vINT			AreaUserCount;		/* current number of users  */
  vLONG 		AreaStatusFlags;	/* area status flags	    */
  vLONG 		AreaRemBackupTime;	/* remaining backup time (seconds) */

  vCHAR 		AreaName[128];		/* area filename */
}
vC16_AreaInfo;

/* user information structure */
typedef struct
{
  vINT			InfoSize;		/* size of information block */

  vINT			ID;			/* user id		    */
  vINT			Number; 		/* user number		    */
  vINT			Protocol;		/* network protocol	    */

  vLONG 		FirstReq;		/* seconds since login	    */
  vDATE 		FirstReqDate;		/* login date		    */
  vTIME 		FirstReqTime;		/* login time		    */
  vLONG 		LastReq;		/* seconds since last req.  */
  vDATE 		LastReqDate;		/* last request date	    */
  vTIME 		LastReqTime;		/* last request time	    */

  vCHAR 		Name[24];		/* user name		    */
  vCHAR 		SysName[64];		/* machine name 	    */
  vCHAR 		SysNameIP[64];		/* host name		    */
  vCHAR 		Address[24];		/* network address	    */

  vCHAR 		SysAccount[64]; 	/* system account name	    */
  vCHAR 		NetAccount[64]; 	/* network account name     */
}
vC16_UserInfo;

/****************************************************************************/
/*									    */
/*  CALL CONTROL BLOCK							    */
/*									    */
/****************************************************************************/

typedef struct
{
  vPHANDLE		InstHdl;		/* instance handle */

/****************************************************************************/
/*  Argument access							    */
/****************************************************************************/

vERROR (WINAPI * C16_ArgCount)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  vINT* 		aArgCount		/* out: number of arguments */
);

vERROR (WINAPI * C16_ArgInfo)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aPosition,		/* in:	argument number     */
  vINT* 		aType,			/* out: argument type	    */
  vINT* 		aOptions,		/* out: argument options    */
  vINT* 		aLength,		/* out: argument length     */
  vINT* 		aMaxLength,		/* out: max. string size    */
  vINT* 		aElements		/* out: array elements	    */
);

vERROR (WINAPI * C16_ArgRead)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aPosition,		/* in:	argument number     */
  const vINT		aIndex, 		/* in:	array index	    */
  void* 		aData			/* out: argument data	    */
);

vERROR (WINAPI * C16_ArgWrite)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aPosition,		/* in:	argument number     */
  const vINT		aIndex, 		/* in:	array index	    */
  void* 		aData			/* out: argument data	    */
);

/****************************************************************************/
/*  FILE information							    */
/****************************************************************************/

/* retrieve file information by number */
vERROR (WINAPI * C16_FileInfo)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  vC16_FileInfo*	aInfoBlock		/* out: information block   */
);

/* retrieve file information by name */
vERROR (WINAPI * C16_FileInfoByName)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vCHAR*		aFileName,		/* in:	filename	    */
  vC16_FileInfo*	aInfoBlock		/* out: information block   */
);

/****************************************************************************/
/*  SUBRECORD information						    */
/****************************************************************************/

/* retrieve subrecord information */
vERROR (WINAPI * C16_SbrInfo)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vINT		aSbrNo, 		/* in:	subrecord number    */
  vC16_SbrInfo* 	aInfoBlock		/* out: information block   */
);

/* retrieve subrecord information by name */
vERROR (WINAPI * C16_SbrInfoByName)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vCHAR*		aSbrName,		/* in:	subrecord name	    */
  vC16_SbrInfo* 	aInfoBlock		/* out: information block   */
);

/* clear subrecord buffer */
vERROR (WINAPI * C16_SbrClear)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vINT		aSbrNo			/* in:	subrecord number    */
);

/* change subrecord status */
vERROR (WINAPI * C16_SbrSetStatus)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vINT		aSbrNo, 		/* in:	subrecord number    */
  const vBOOL		aNewStatus		/* in:	new status	    */
);

/****************************************************************************/
/*  FIELD information							    */
/****************************************************************************/

/* retrieve field information */
vERROR (WINAPI * C16_FldInfo)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vINT		aSbrNo, 		/* in:	subrecord number    */
  const vINT		aFldNo, 		/* in:	field number	    */
  vC16_FldInfo* 	aInfoBlock		/* out: information block   */
);

/* retrieve field information by name */
vERROR (WINAPI * C16_FldInfoByName)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vCHAR*		aFldName,		/* in:	field name	    */
  vC16_FldInfo* 	aInfoBlock		/* out: information block   */
);

/****************************************************************************/
/*  KEY information							    */
/****************************************************************************/

/* retrieve key information */
vERROR (WINAPI * C16_KeyInfo)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vINT		aKeyNo, 		/* in:	key number	    */
  vC16_KeyInfo* 	aInfoBlock		/* out: information block   */
);

/* retrieve key information by name */
vERROR (WINAPI * C16_KeyInfoByName)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vCHAR*		aKeyName,		/* in:	key name	    */
  vC16_KeyInfo* 	aInfoBlock		/* out: information block   */
);

/* retrieve key field information */
vERROR (WINAPI * C16_KeyFldInfo)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vINT		aKeyNo, 		/* in:	key number	    */
  const vINT		aKeyFldNo,		/* in:	key field number    */
  vC16_KeyFldInfo*	aInfoBlock		/* out: information block   */
);

/****************************************************************************/
/*  LINK information							    */
/****************************************************************************/

/* retrieve link information */
vERROR (WINAPI * C16_LinkInfo)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vINT		aLinkNo,		/* in:	link number	    */
  vC16_LinkInfo*	aInfoBlock		/* out: information block   */
);

/* retrieve link information by name */
vERROR (WINAPI * C16_LinkInfoByName)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vCHAR*		aLinkName,		/* in:	link name	    */
  vC16_LinkInfo*	aInfoBlock		/* out: information block   */
);

/* retrieve link field information */
vERROR (WINAPI * C16_LinkFldInfo)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vINT		aLinkNo,		/* in:	link number	    */
  const vINT		aLinkFldNo,		/* in:	link field number   */
  vC16_LinkFldInfo*	aInfoBlock		/* out: information block   */
);

/****************************************************************************/
/*  data exchange							    */
/****************************************************************************/

/* set character format defaults */
vERROR (WINAPI * C16_SetCharDefs)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  vCHAR 		aDecimalSep,		/* in:	decimal separator   */
  vCHAR 		aThousandsSep,		/* in:	thousands separator */
  vCHAR 		aDateSep,		/* in:	date separator	    */
  vCHAR 		aTimeSep1,		/* in:	time separator 1    */
  vCHAR 		aTimeSep2,		/* in:	time separator 2    */
  vCHAR 		aCharTRUE,		/* in:	char for TRUE value */
  vCHAR 		aCharFALSE,		/* in:	char for FALSE value */
  vFLAGS		aCharSetConversion	/* in:	charset conv. type  */
);

/* read/write field data */
vERROR (WINAPI * C16_FldData)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vINT		aSbrNo, 		/* in:	subrecord number    */
  const vINT		aFldNo, 		/* in:	field number	    */
  void* 		aFldBuf,		/* in/out: field data	    */
  const vBOOL		aWriteFlag		/* in:	write data to field */
);

/* read/write field data as chars */
vERROR (WINAPI * C16_FldDataChar)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vINT		aSbrNo, 		/* in:	subrecord number    */
  const vINT		aFldNo, 		/* in:	field number	    */
  vCHAR*		aFldBuf,		/* in/out: field data	    */
  const vFLAGS		aFormatFlags,		/* in:	format options	    */
  const vINT		aPostcomma,		/* in:	postcomma digits    */
  const vBOOL		aWriteFlag		/* in:	write data to field */
);

/* read/write field data (raw alpha) */
vERROR (WINAPI * C16_FldDataRaw)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vINT		aSbrNo, 		/* in:	subrecord number    */
  const vINT		aFldNo, 		/* in:	field number	    */
  void* 		aFldBuf,		/* in/out: field data	    */
  const vBOOL		aWriteFlag		/* in:	write data to field */
);

/* register external field buffer */
vERROR (WINAPI * C16_RegExtFld)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vINT		aSbrNo, 		/* in:	subrecord number    */
  const vINT		aFldNo, 		/* in:	field number	    */
  const vFLAGS		aExcFlags,		/* in:	data exchange flags */
  const vFLAGS		aFormatFlags,		/* in:	format options	    */
  const vINT		aPostcomma,		/* in:	postcomma digits    */
  void* 		aExtBuffer		/* in:	external field buffer */
);

/* transfer complete record */
vERROR (WINAPI * C16_ExcFileData)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vPHANDLE	aBuffHdl,		/* in:	buffer handle	    */
  const vBOOL		aWriteFlag		/* in:	write data	    */
);

/* transfer subrecord */
vERROR (WINAPI * C16_ExcSbrData)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vINT		aSbrNo, 		/* in:	subrecord number    */
  const vBOOL		aWriteFlag		/* in:	write data	    */
);

/* transfer single field */
vERROR (WINAPI * C16_ExcFldData)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vINT		aSbrNo, 		/* in:	subrecord number    */
  const vINT		aFldNo, 		/* in:	field number	    */
  const vBOOL		aWriteFlag		/* in:	write data	    */
);

/* set automatic data exchange option */
vERROR (WINAPI * C16_SetExcMode)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vBOOL		aAutoFlag		/* in:	enable/disable flag */
);

/****************************************************************************/
/*  record information							    */
/****************************************************************************/

/* retrieve record information */
vERROR (WINAPI * C16_RecInfo)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vINT		aInfoType,		/* in:	information type    */
  const vINT		aKeyNo, 		/* in:	key number (opt.)   */
  vINT* 		aResult 		/* out: information result  */
);

/* retrieve linked record information */
vERROR (WINAPI * C16_RecLinkInfo)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aSrcFileNo,		/* in:	source filenumber   */
  const vINT		aDstFileNo,		/* in:	destin. filenumber  */
  const vINT		aLinkNo,		/* in:	link number	    */
  const vINT		aInfoType,		/* in:	information type    */
  const vINT		unused, 		/* in:	reserved (0)	    */
  vINT* 		aResult 		/* out: information result  */
);

/****************************************************************************/
/*  record operations							    */
/****************************************************************************/

/* read record */
vERROR (WINAPI * C16_RecRead)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vINT		aKeyNo, 		/* in:	key number	    */
  const vFLAGS		aFlags, 		/* in:	operation flags     */
  const vLONG		aAddInfo		/* in:	additional info     */
);

/* read linked record */
vERROR (WINAPI * C16_RecLink)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aSrcFileNo,		/* in:	source filenumber   */
  const vINT		aDstFileNo,		/* in:	destin. filenumber  */
  const vINT		aLinkNo,		/* in:	link number	    */
  const vFLAGS		aFlags, 		/* in:	operation flags     */
  const vINT		unused, 		/* in:	reserved (0)	    */
  const vINT		aLinkPos		/* in:	link position (opt.)*/
);

/* insert new record */
vERROR (WINAPI * C16_RecInsert)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vFLAGS		aFlags			/* in:	operation flags     */
);

/* update record */
vERROR (WINAPI * C16_RecReplace)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vFLAGS		aFlags			/* in:	operation flags     */
);

/* delete single record */
vERROR (WINAPI * C16_RecDelete)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vFLAGS		aFlags			/* in:	operation flags     */
);

/* delete all records */
vERROR (WINAPI * C16_RecDeleteAll)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo 		/* in:	filenumber	    */
);

/****************************************************************************/
/*  record buffer operations						    */
/****************************************************************************/

/* clear record buffer - buffer handle is optional (0 if not used) */
vERROR (WINAPI * C16_RecBufClear)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  const vPHANDLE	aBuffHdl		/* in:	buffer handle	    */
);

/* copy record buffer - buffer handles are optional (0 if not used) */
vERROR (WINAPI * C16_RecBufCopy)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aSrcFileNo,		/* in:	source filenumber   */
  const vINT		aDstFileNo,		/* in:	destin. filenumber  */
  const vPHANDLE	aSrcBuffHdl,		/* in:	source buf. handle  */
  const vPHANDLE	aDstBuffHdl		/* in:	destin. buf. handle */
);

/* compare record buffers - buffer handles are optional (0 if not used) */
vERROR (WINAPI * C16_RecBufCompare)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo1,		/* in:	filenumber 1	    */
  const vINT		aFileNo2,		/* in:	filenumber 2	    */
  const vPHANDLE	aBuffHdl1,		/* in:	buffer handle 1     */
  const vPHANDLE	aBuffHdl2		/* in:	buffer handle 2     */
);

/* create temporary record buffer */
vERROR (WINAPI * C16_RecBufCreate)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aFileNo,		/* in:	filenumber	    */
  vPHANDLE*		aBuffHdl		/* out: new buffer handle   */
);

/* destroy temporary record buffer */
vERROR (WINAPI * C16_RecBufDestroy)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vPHANDLE	aBuffHdl		/* in:	buffer handle	    */
);

/****************************************************************************/
/*  transactions							    */
/****************************************************************************/

/* begin transaction */
vERROR (WINAPI * C16_DtaBegin)
(
  const vPHANDLE	aInstHdl		/* in:	instance handle     */
);

/* commit transaction */
vERROR (WINAPI * C16_DtaCommit)
(
  const vPHANDLE	aInstHdl		/* in:	instance handle     */
);

/* rollback transaction */
vERROR (WINAPI * C16_DtaRollback)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vBOOL		aAllLevels		/* in:	complete rollback   */
);

/****************************************************************************/
/*  text operations							    */
/****************************************************************************/

/* create text buffer */
vERROR (WINAPI * C16_TextOpen)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  vPHANDLE*		aTextHdl		/* out: new text handle     */
);

/* close text buffer */
vERROR (WINAPI * C16_TextClose)
(
  const vPHANDLE	aTextHdl		/* in:	text handle	    */
);

/* create text */
vERROR (WINAPI * C16_TextCreate)
(
  const vPHANDLE	aTextHdl,		/* in:	text handle	    */
  const vCHAR*		aTextName,		/* in:	text to create	    */
  const vBOOL		aLockText		/* in:	lock new text	    */
);

/* copy text */
vERROR (WINAPI * C16_TextCopy)
(
  const vPHANDLE	aTextHdl,		/* in:	text handle	    */
  const vCHAR*		aTextName,		/* in:	text to copy	    */
  const vCHAR*		aTextNameNew		/* in:	new text name	    */
);

/* rename text */
vERROR (WINAPI * C16_TextRename)
(
  const vPHANDLE	aTextHdl,		/* in:	text handle	    */
  const vCHAR*		aTextName,		/* in:	text to rename	    */
  const vCHAR*		aTextNameNew		/* in:	new text name	    */
);

/* delete text */
vERROR (WINAPI * C16_TextDelete)
(
  const vPHANDLE	aTextHdl,		/* in:	text handle	    */
  const vCHAR*		aTextName		/* in:	text to delete	    */
);

/* read text header */
vERROR (WINAPI * C16_TextRead)
(
  const vPHANDLE	aTextHdl,		/* in:	text handle	    */
  const vCHAR*		aTextName,		/* in:	text name to read   */
  const vFLAGS		aFlags, 		/* in:	operation flags     */
  vC16_TextInfo*	aTextInfo		/* out: text information    */
);

/* read text data */
vERROR (WINAPI * C16_TextReadData)
(
  const vPHANDLE	aTextHdl,		/* in:	text handle	    */
  const void*		aBuffer,		/* in:	text data buffer    */
  const vFLAGS		aFlags, 		/* in:	operation flags     */
  const vINT		aBufferSize,		/* in:	size of aBuffer     */
  vINT* 		aTextSize,		/* out: size of text	    */
  vINT* 		aTextLines		/* out: number of text lines */
);

/* write text header */
vERROR (WINAPI * C16_TextWrite)
(
  const vPHANDLE	aTextHdl,		/* in:	text handle	    */
  const vFLAGS		aFlags, 		/* in:	operation flags     */
  vC16_TextInfo*	aTextInfo		/* in:	text information    */
);

/* write text data */
vERROR (WINAPI * C16_TextWriteData)
(
  const vPHANDLE	aTextHdl,		/* in:	text handle	    */
  const void*		aBuffer,		/* in:	text data buffer    */
  const vFLAGS		aFlags, 		/* in:	operation flags     */
  const vINT		aTextSize		/* in:	size of text	    */
);

/****************************************************************************/
/*  procedures								    */
/****************************************************************************/

/* define procedure argument */
vERROR (WINAPI * C16_ProcArgument)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vBYTE		aArgType,		/* in:	argument type	    */
  void* 		aArgVal,		/* in:	argument value	    */
  const vBOOL		aVarArg,		/* in:	pass by reference   */
  const vBYTE		aMaxLen 		/* in:	maximum string len  */
);

/* call procedure */
vERROR (WINAPI * C16_ProcCall)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vCHAR*		aProcName		/* in:	procedure name	    */
);

/* retrieve procedure result */
vERROR (WINAPI * C16_ProcResult)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  vBYTE*		aResultType,		/* out: result type	    */
  void**		aResultData		/* out: result data	    */
);

/****************************************************************************/
/*  maintainance & information						    */
/****************************************************************************/

/* start/stop server backup events */
vERROR (WINAPI * C16_ServerBackup)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  const vINT		aMinutes		/* in:	backup event duration */
);

/* query interface informations */
vERROR (WINAPI * C16_QueryPgifInfo)
(
  const vPHANDLE	aPgifHdl,		/* in:	module handle	    */
  vC16_PgifInfo*	aInfoBlock		/* out: information block   */
);

/* query server informations */
vERROR (WINAPI * C16_QueryServerInfo)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle     */
  vC16_ServerInfo*	aInfoBlock		/* out: information block   */
);

/* query data area informations */
vERROR (WINAPI * C16_QueryAreaInfo)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle */
  vC16_AreaInfo*	aInfoBlock		/* out: information block   */
);

/* query user informations */
vERROR (WINAPI * C16_QueryUserInfo)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle */
  const vINT		aUserID,		/* in:	user ID */
  const vBOOL		aNextID,		/* in:	read next user ID */
  vC16_UserInfo*	aInfoBlock		/* out: information block   */
);

/* clear user */
vERROR (WINAPI * C16_UserClear)
(
  const vPHANDLE	aInstHdl,		/* in:	instance handle */
  const vINT		aUserID,		/* in:	user ID */
  const vINT		aUserNo,		/* in:	user number */
  const vCHAR*		aServerPassword 	/* in:	server password */
);

}
vC16_CCB;

#ifdef __cplusplus
};
#endif

#endif

